/////////////////////////////////////////////////////////////////////////////
// Name:        epub_property.h
// Purpose:     EpProperty objects represent name/value pairs.
// Author:      Julian Smart
// Modified by:
// Created:     2002-12-11
// RCS-ID:      $Id: EpProperty.h,v 1.2 2008/01/18 13:11:24 anthemion Exp $
// Copyright:   (c) Julian Smart
// Licence:     New BSD License
/////////////////////////////////////////////////////////////////////////////

#ifndef _EPUB_PROPERTY_H_
#define _EPUB_PROPERTY_H_

#include "wx/variant.h"

/*!
 * A property/name pair, with other specialised members for
 * storing window/sizer information.
 */

class EpProperty: public wxObject
{
    DECLARE_CLASS(EpProperty)
public:
    EpProperty()
    {
        m_styleValue = 0;
        m_show = TRUE;
    }

    EpProperty(const EpProperty& property) { Copy(property); }
    EpProperty(const wxString& descr, const wxVariant& variant,
        const wxString& editorType = wxEmptyString,
        const wxString& groupName = wxEmptyString,
        const wxString& exclusivityGroupName = wxEmptyString,
        long styleValue = 0)
    {
        m_description = descr;
        m_variant = variant;
        m_editorType = editorType;
        m_groupName = groupName;
        m_exclusivityGroupName = exclusivityGroupName;
        m_styleValue = styleValue;
        m_show = TRUE;
    }

    EpProperty(const wxString& name, const wxString& value = wxEmptyString)
    {
        m_variant.SetName(name);
        m_variant = value;
        m_show = TRUE;
    }

    void operator= (const EpProperty& property) { Copy(property); }
    void Copy(const EpProperty& property)
    {
        m_variant = property.m_variant;
        m_groupName = property.m_groupName;
        m_exclusivityGroupName = property.m_exclusivityGroupName;
        m_styleValue = property.m_styleValue;
        m_editorType = property.m_editorType;
        m_description = property.m_description;
        m_choices = property.m_choices;
        m_show = property.m_show;
        m_parentName = property.m_parentName;
        m_translatedName = property.m_translatedName;
    }

    bool operator== (const EpProperty& property) const
    {
        return ((m_variant == property.m_variant) &&
            (m_groupName == property.m_groupName) &&
            (m_parentName == property.m_parentName) &&
            (m_exclusivityGroupName == property.m_exclusivityGroupName) &&
            (m_styleValue == property.m_styleValue) &&
            (m_editorType == property.m_editorType) &&
            (m_description == property.m_description) &&
            (m_show == property.m_show) &&
            (m_translatedName == property.m_translatedName) &&
            (m_choices == property.m_choices));
    }

    bool operator!= (const EpProperty& property) const
    {
        return !((*this) == property);
    }

    inline const wxString& GetName() const { return m_variant.GetName(); }
    inline const wxString& GetParentName() const { return m_parentName; }
    inline wxString GetValue() const { return m_variant.GetString(); }
    inline wxVariant& GetVariant() { return m_variant; }
    inline const wxVariant& GetVariant() const { return m_variant; }
    inline const wxString& GetEditorType() const { return m_editorType; }
    inline const wxString& GetGroupName() const { return m_groupName; }
    inline const wxString GetCategory() const { return m_category; }
    inline const wxString& GetExclusivityGroupName() const { return m_exclusivityGroupName; }
    inline long GetStyleValue() const { return m_styleValue; }
    inline const wxArrayString& GetChoices() const { return m_choices; }
    inline const wxString& GetDescription() const { return m_description; }
    inline const wxString& GetMinVersion() const { return m_minVersion; }
    inline const wxString& GetMaxVersion() const { return m_maxVersion; }
    
    inline void SetName(const wxString& name) { m_variant.SetName(name); }
    inline void SetParentName(const wxString& name) { m_parentName = name; }
    inline void SetValue(const wxString& value) { m_variant = value; }
    inline void SetValue(const wxVariant& value) { m_variant = value; }
    inline void SetEditorType(const wxString& type) { m_editorType = type; }
    inline void SetGroupName(const wxString& groupName) { m_groupName = groupName; }
    inline void SetCategory(const wxString& category) { m_category = category; }
    inline void SetExclusivityGroupName(const wxString& name) { m_exclusivityGroupName = name; }
    inline void SetStyleValue(long style) { m_styleValue = style; }
    inline void SetChoices(const wxArrayString& choices) { m_choices = choices; }
    inline void SetDescription(const wxString& descr) { m_description = descr; }
    inline void SetMinMaxVersion(const wxString& minVersion, const wxString& maxVersion)
        { m_minVersion = minVersion; m_maxVersion = maxVersion; }
    inline void Show(bool show) { m_show = show; }
    inline bool IsShown() const { return m_show; }

    // Get/set the visible translation, if any
    const wxString& GetTranslatedName() const { return m_translatedName; }
    void SetTranslatedName(const wxString& name) { m_translatedName = name; }

    // Get display name - translation if available, or actual
    wxString GetDisplayName() const { return m_translatedName.IsEmpty() ? GetName() : GetTranslatedName(); }

    // Is this property applicable to the given wxWidgets version?
    bool IsApplicableToVersion(const wxString& useVersion) const;

    // The name and value
    wxVariant   m_variant;

    // The visible translation, if any
    wxString    m_translatedName;

    // The editor type name (e.g. "file")
    // used to choose an editor.
    wxString    m_editorType;

    // The group name, e.g. window-style
    wxString    m_groupName;

    // The parent property name. Can be dot-separated if more than one.
    wxString    m_parentName;

    // Exclusivity group name, e.g. vertical-alignment
    wxString    m_exclusivityGroupName;

    // Style value, if any.
    long        m_styleValue;

    // Array of choices
    wxArrayString   m_choices;

    // Description
    wxString        m_description;

    // Min wxWidgets version appropriate for this property/style
    wxString        m_minVersion;

    // Max wxWidgets version appropriate for this property/style
    wxString        m_maxVersion;

    // Category for this property
    wxString        m_category;

    // Whether to show or hide (e.g. not properly initialized)
    bool            m_show;
};

// A class to manage properties
class EpProperties: public wxObject
{
    DECLARE_CLASS(EpProperties)
public:
    EpProperties() {}
    EpProperties(const EpProperties& properties) { Copy(properties); }
    ~EpProperties() { Clear(); }

    void operator = (const EpProperties& properties) { Clear(); Copy(properties); }
    bool operator == (const EpProperties& properties) ;
    void Copy(const EpProperties& properties);

    inline const wxList& GetList() const { return m_list; }

    inline size_t GetCount() const { return m_list.GetCount(); }

    EpProperty* AddProperty(EpProperty* property, const wxString& insertAfter = wxEmptyString);
    void SetProperty(const wxString& name, const wxString& value);
    void SetProperty(const wxString& name, long value);
    void SetProperty(const wxString& name, int value) { SetProperty(name, (long) value); }
    void SetProperty(const wxString& name, double value);
    void SetProperty(const wxString& name, bool value);
    void SetProperty(const wxString& name, const wxVariant& value);
    void RemoveProperty(EpProperty* property);
    void DeleteProperty(EpProperty* property);
    void DeleteProperty(const wxString& name);
    void DeleteProperty(size_t i);
    EpProperty* FindProperty(const wxString& name) const;
    EpProperty* FindPropertyByDisplayName(const wxString& name);
    EpProperty* FindOrCreateProperty(const wxString& name);
    wxString FindPropertyValueString(const wxString& name) const;
    bool FindPropertyValueBool(const wxString& name) const;
    long FindPropertyValueLong(const wxString& name) const;
    double FindPropertyValueDouble(const wxString& name) const;
    wxVariant FindPropertyValue(const wxString& name) const;
    EpProperty* GetNth(int i) const;
    EpProperty* GetProperty(int i) const { return GetNth(i); }
    
    // Remove any spurious properties that need garbage
    // collecting.
    void RemoveHiddenProperties();

// Dealing with styles

    /// Combine the styles of all selected properties
    /// with this group name. If useVersion is not empty,
    /// use this wxWidgets version to determine whether the style
    /// is appropriate or not.
    long CombineStyles(const wxString& groupName, const wxString& useVersion = wxEmptyString);

    /// Combine the styles of all selected properties
    /// with this group name. If useVersion is not empty,
    /// use this wxWidgets version to determine whether the style
    /// is appropriate or not.
    wxString CombineStylesString(const wxString& groupName, const wxString& useVersion = wxEmptyString);

    void Clear();

private:
    wxList      m_list;
};

#endif
    // _EPUB_PROPERTY_H_

